import datetime

class Channel:
    
    def __init__(self, name):
        self.name = name
        self.subscribers = []
        self.videos = []
        
    def subscribe(self, user):
        self.subscribers.append(user)
        user.channels.append(self)
    
    def unsubscribe(self, user):
        self.subscribers.remove(user)
        user.channels.remove(self)
        
    def publish(self, titre_de_video):
        print(f"Le canal {self.name} publie '{titre_de_video}'")
        date = datetime.datetime.now()
        self.videos.append({"titre": titre_de_video,
                            "date": date})
        self.notifySubscribers()
    
    def notifySubscribers(self):
        # Pour chaque abonné?, dé?clencher la mé?thode 'actualiser' chez l'abonné?
        for user in self.subscribers:
            user.actualiser()
    
class User:

    def __init__(self, name):
        
        self.name = name
        self.channels = []
    
    def __repr__(self):
        return f"<User {self.name}>"
    
    def actualiser(self):
        
        print(f"Je suis dans le actualiser() de {self.name}")
        
        # 1. Lister toutes les vidé?os des channels où? on est abonné?
        
        # Initialiser une liste temporaire
        toutes_les_videos = []
        # Pour chaque channel
        for channel in self.channels:
            # Pour chaque video du channel
            for video in channel.videos:
                # Garder la video dans une liste temporaire
                video = video.copy()
                video["channel"] = channel.name
                toutes_les_videos.append(video)
        
        print(f"Toutes les videos qui inté?resse {self.name} : {toutes_les_videos}")
        
        # 2. Trier ces vidé?os par date
        # et garder les 3 les plus ré?centes
        toutes_les_videos = sorted(toutes_les_videos, key=lambda v: v["date"])
        videos_a_montrer = toutes_les_videos[-3:]
        
        # 3. é?crire dans un fichier latest_videos_for_{user}.txt
        # la liste des videos (nom du canal - titre de la video)
        with open(f"latest_videos_for_{self.name}.txt", "w") as f:
            for video in videos_a_montrer:
                f.write(video["channel"] + " : " + video["titre"] + "\n")


arte = Channel("ARTE")
cestpassorcier = Channel("C'est pas sorcier")
videodechat = Channel("Video de chat")

alice = User("alice")
bob = User("bob")
charlie = User("charlie")

arte.subscribe(alice)
cestpassorcier.subscribe(alice)
cestpassorcier.subscribe(bob)
videodechat.subscribe(bob)
videodechat.subscribe(charlie)


cestpassorcier.publish("Le systè?me solaire")
arte.publish("La grenouille, un animal extraordinaire")
cestpassorcier.publish("Le gé?nie des fourmis")
videodechat.publish("Video de chat qui fait miaou")
cestpassorcier.publish("Les chateaux forts")
